/**********************************************************************************************
    Copyright (C) 2017 Oliver Eichler <oliver.eichler@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/

#ifndef CSHELLCMD_H
#define CSHELLCMD_H

#include <QString>
#include <QStringList>

class CShellCmd
{
public:
    CShellCmd(const QString& cmd, const QStringList& args);
    virtual ~CShellCmd() = default;

    const QString& getCmd() const
    {
        return cmd;
    }

    const QStringList& getArgs() const
    {
        return args;
    }

private:
    QString cmd;
    QStringList args;
};

#endif //CSHELLCMD_H

